SystemOrganization addCategory: #'Volunti-Urn-Model'!
SystemOrganization addCategory: #'Volunti-Urn-View'!
SystemOrganization addCategory: #'Volunti-Urn-Tests'!

TestCase subclass: #UrnTest
	instanceVariableNames: 'full empty urnWithDuplicates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Tests'!

!UrnTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 19:51'!
electionId
	^ 10929! !

!UrnTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 19:48'!
numberOfBallots
	^ 1000! !

!UrnTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 19:47'!
numberOfPossibleVotes
	^ 5! !

!UrnTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/6/2012 15:03'!
numberOfVoters
	^ 20! !

!UrnTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 3/6/2012 15:14'!
setUp
	| random |
	empty := VOUrn new.
	random := SecureRandom new.
	urnWithDuplicates := VOUrn new id: self electionId.
	1 to: self numberOfVoters do: [ :id | 
		| ballot |
		ballot := VOBallot numberOfPossibleVotes: 5.
		ballot
			give: 1 additionalPointsTo: (random nextInt: self numberOfPossibleVotes);
			electionId: self electionId;
			voterId: id.
		urnWithDuplicates addBallot: ballot ].
	self numberOfBallots
		timesRepeat: [ 
			| ballot |
			ballot := VOBallot numberOfPossibleVotes: 5.
			ballot
				give: 1 additionalPointsTo: (random nextInt: self numberOfPossibleVotes);
				electionId: self electionId;
				voterId: (random nextFrom: 1 to: self numberOfVoters).
			urnWithDuplicates addBallot: ballot ].
	full := VOUrn new id: self electionId.
	self numberOfBallots
		timesRepeat: [ 
			| ballot |
			ballot := VOBallot numberOfPossibleVotes: 5.
			ballot
				give: 1 additionalPointsTo: (random nextInt: self numberOfPossibleVotes);
				electionId: self electionId.
			full addBallot: ballot ]! !

!UrnTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 19:49'!
testAddBallot
	self assert: empty isEmpty.
	empty
		addBallot: (VOBallot numberOfPossibleVotes: 5);
		addBallot: (VOBallot numberOfPossibleVotes: 5).
	self assert: empty tally = 2.
	self assert: full tally = self numberOfBallots! !

!UrnTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 19:53'!
testCompressed
	"to remove"
	! !

!UrnTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 8/20/2012 11:51'!
testWithoutDuplicate
	self assert: urnWithDuplicates withoutDuplicate size = (self numberOfVoters + self numberOfBallots)! !

TestCase subclass: #VOBallotTest
	instanceVariableNames: 'ballot1 ballot2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Tests'!

!VOBallotTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 15:29'!
numberOfPossibleVotes
	^ 100! !

!VOBallotTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 2/29/2012 15:58'!
setUp
	ballot1 := VOBallot numberOfPossibleVotes: self numberOfPossibleVotes.
	ballot2 := VOBallot numberOfPossibleVotes: self numberOfPossibleVotes! !

!VOBallotTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 16:24'!
testGiveAdditionalPoints
	ballot1 give: 1 additionalPointsTo: 20.
	self shouldnt: [ ballot1 validate ] raise: WAValidationNotification.
	ballot1 give: 1 additionalPointsTo: 30.
	self should: [ ballot1 validate ] raise: WAValidationNotification! !

VOBallotTest subclass: #VOKOutOfNBallotTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Tests'!

!VOKOutOfNBallotTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:13'!
maxNumberOfAllowedVotes
	^ 20! !

!VOKOutOfNBallotTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 2/29/2012 16:13'!
setUp
	ballot1 := VOKOutOfNBallot
		numberOfPossibleVotes: self numberOfPossibleVotes
		maxNumberOfAllowedVotes: self maxNumberOfAllowedVotes.
	ballot2 := VOKOutOfNBallot
		numberOfPossibleVotes: self numberOfPossibleVotes
		maxNumberOfAllowedVotes: self maxNumberOfAllowedVotes! !

!VOKOutOfNBallotTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 16:24'!
testGiveAdditionalPoints
	1 to: self maxNumberOfAllowedVotes do: [ :i | ballot1 give: 1 additionalPointsTo: i ].
	self shouldnt: [ ballot1 validate ] raise: WAValidationNotification.
	ballot1 give: 1 additionalPointsTo: self maxNumberOfAllowedVotes + 1.
	self should: [ ballot1 validate ] raise: WAValidationNotification! !

VOKOutOfNBallotTest subclass: #VOMPtKOutOfNBallotTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Tests'!

!VOMPtKOutOfNBallotTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:23'!
pointsToDistribute
	^ 30! !

!VOMPtKOutOfNBallotTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 2/29/2012 19:40'!
setUp
	ballot1 := VOMPtKOutOfNBallot numberOfPossibleVotes: 5 maxNumberOfAllowedVotes: 2 pointsToDistribute: 10.
	ballot2 := VOMPtKOutOfNBallot numberOfPossibleVotes: 5 maxNumberOfAllowedVotes: 2 pointsToDistribute: 10.! !

!VOMPtKOutOfNBallotTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 19:41'!
testGiveAdditionalPoints
	ballot1
		give: 4 additionalPointsTo: 1;
		give: 2 additionalPointsTo: 2;
		give: 1 additionalPointsTo: 3;
		give: 0 additionalPointsTo: 4;
		give: 3 additionalPointsTo: 5.
	self shouldnt: [ ballot1 validate ] raise: WAValidationNotification.
	ballot2
		give: 2 additionalPointsTo: 1;
		give: 0 additionalPointsTo: 2;
		give: 0 additionalPointsTo: 3;
		give: 4 additionalPointsTo: 4;
		give: 5 additionalPointsTo: 5.
	self should: [ ballot2 validate ] raise: WAValidationNotification! !

VOBallotTest subclass: #VOMBBallotTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Tests'!

VOMBBallotTest subclass: #VOMBAdvancedBallotTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Tests'!

!VOMBAdvancedBallotTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 5/8/2012 11:01'!
setUp
	ballot1 := VOMBAlternativeBallot numberOfPossibleVotes: self numberOfPossibleVotes.
	ballot2 := VOMBAlternativeBallot numberOfPossibleVotes: self numberOfPossibleVotes! !

!VOMBAdvancedBallotTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 16:52'!
testGiveAdditionalPoints! !

!VOMBBallotTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 2/29/2012 16:37'!
setUp
	ballot1 := VOMBBallot numberOfPossibleVotes: self numberOfPossibleVotes.
	ballot2 := VOMBBallot numberOfPossibleVotes: self numberOfPossibleVotes! !

!VOMBBallotTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 16:43'!
testGiveAdditionalPoints
	ballot1 give: 2390 additionalPointsTo: 1.
	ballot1 give: 3245 additionalPointsTo: 2.
	ballot1 give: 3245 additionalPointsTo: 3.
	self assert: (ballot1 hasDuplicatesBefore: 3).
	ballot1 give: 4598 additionalPointsTo: 3.
	self deny: (ballot1 hasDuplicatesBefore: 3).
	self should: [ ballot1 validate ] raise: WAValidationNotification.
	4 to: self numberOfPossibleVotes do: [ :i | ballot1 give: i additionalPointsTo: i ].
	self shouldnt: [ ballot1 validate ] raise: WAValidationNotification! !

!VOMBBallotTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 16:35'!
testValidate
"	ballot2 addVote: (VOVote id: 1 secret: (ballot2 secrets atRandom)).
	self should: [ ballot2 validate ] raise: WAValidationNotification.
	ballot2 addVoterSecret: voterSecret.
	ballot2 addVote: (VOVote id: 1 secret: voterSecret).
	self shouldnt: [ ballot2 validate ] raise: WAValidationNotification"! !

WATask subclass: #VOMBBallotTask
	instanceVariableNames: 'model choices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-View'!

VOMBBallotTask subclass: #VOAlternativeMBBallotTask
	instanceVariableNames: 'availableSecrets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-View'!

!VOAlternativeMBBallotTask class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/4/2012 08:46'!
model: aMBBallot list: choices availableSecrets: aList
	self assert: aList notNil.
	^ (self model: aMBBallot list: choices) availableSecrets: aList! !

!VOAlternativeMBBallotTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/8/2012 11:07'!
availableSecrets: list
	availableSecrets := list! !

!VOAlternativeMBBallotTask methodsFor: 'tasks' stamp: 'NicolasVanderAuwera 8/23/2012 10:12'!
fillChoice: choice
	^ availableSecrets 
		remove: (self call: (VOAlternativeMBVoteEditor choice: choice availableSecrets: availableSecrets))
		ifAbsent: [  ]! !

!VOMBBallotTask class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/6/2012 08:12'!
model: aMBBallot list: choices
	^ self new
		model: aMBBallot;
		choices: choices;
		yourself! !

!VOMBBallotTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/6/2012 08:12'!
choices
	^ choices! !

!VOMBBallotTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/6/2012 08:12'!
choices: anObject
	choices := anObject! !

!VOMBBallotTask methodsFor: 'tasks' stamp: 'NicolasVanderAuwera 5/8/2012 13:18'!
fillChoice: choice
	^ self call: (VOMBVoteEditor choice: choice)! !

!VOMBBallotTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 5/8/2012 13:18'!
go
	(1 to: model numberOfPossibleVotes)
		with: choices
		do: [ :i :choice | model give: (self fillChoice: choice) additionalPointsTo: i ]! !

!VOMBBallotTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/3/2012 13:43'!
model
	^ model! !

!VOMBBallotTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/3/2012 13:43'!
model: anObject
	model := anObject! !

WAComponent subclass: #VOBallotView
	instanceVariableNames: 'model choices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-View'!

!VOBallotView class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/5/2012 08:56'!
model: aBallot choices: choicesList
	^ self new
		model: aBallot;
		choices: choicesList;
		yourself! !

!VOBallotView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/5/2012 08:56'!
choices
	^ choices! !

!VOBallotView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/5/2012 08:56'!
choices: anObject
	choices := anObject! !

!VOBallotView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/5/2012 08:11'!
model
	^ model! !

!VOBallotView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/5/2012 08:12'!
model: aBallot
	model := aBallot! !

!VOBallotView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 1/23/2012 09:03'!
renderActionsOn: html
	self renderVoteButtonOn: html.
	self renderBackAnchorOn: html
	! !

!VOBallotView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 1/23/2012 09:06'!
renderBackAnchorOn: html
	html anchor
		callback: [ self answer: nil ];
		with: 'Back'! !

!VOBallotView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 1/8/2012 09:09'!
renderContentOn: html
	self renderNamesAndVotesOn: html.
	self renderActionsOn: html! !

!VOBallotView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 3/5/2012 14:59'!
renderNamesAndVotesOn: html
	html
		table: [ 
			choices
				with: self model votes
				do: [ :choice :vote | 
					html
						tableRow: [ 
							html tableData: choice name.
							html tableData: vote ] ] ]! !

!VOBallotView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 3/5/2012 14:37'!
renderTitleOn: html
	html heading
		level: 1;
		with: self title! !

!VOBallotView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 1/23/2012 09:03'!
renderVoteButtonOn: html
	html form: [ 
			html submitButton
				callback: [ self answer: self model ];
				with: 'Vote' ]
	! !

!VOBallotView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 13:47'!
rows
	^ #(votes)! !

!VOBallotView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/5/2012 14:37'!
title
	^ 'Review your choices'! !

WAComponent subclass: #VOMBVoteEditor
	instanceVariableNames: 'choice number error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-View'!

VOMBVoteEditor subclass: #VOAlternativeMBVoteEditor
	instanceVariableNames: 'availableSecrets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-View'!

!VOAlternativeMBVoteEditor class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/23/2012 10:03'!
choice: aChoice availableSecrets: aCollection
	| editor |
	editor := self choice: aChoice.
	editor availableSecrets: aCollection.
	^ editor! !

!VOAlternativeMBVoteEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/23/2012 09:39'!
availableSecrets: list
	availableSecrets := list! !

!VOAlternativeMBVoteEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/8/2012 14:05'!
renderAvailableSecretsOn: html
	html div
		class: 'secrets ui-corner-all';
		with: [ availableSecrets do: [ :each | html image url: each ] ]! !

!VOAlternativeMBVoteEditor methodsFor: 'rendering-parts' stamp: 'NicolasVanderAuwera 8/3/2012 13:58'!
renderDescriptionOn: html
	html
		paragraph: [ 
			html text: 'Please select the image you want to attribute to the candidate: '.
			html strong: choice name ]! !

!VOAlternativeMBVoteEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/23/2012 10:04'!
renderFormOn: html
	| group image |
	group := html radioGroup.
	html
		form: [ 
			html
				unorderedList: [ 
					availableSecrets
						do: [ :each | 
							html
								listItem: [ 
									html
										fieldSet: [ 
											html div
												class: 'secrets';
												with: [ 
															html
																span: [ 
																	group radioButton
																		id: html nextId;
																		callback: [ image := each ].
																	html label
																		for: html lastId;
																		with: [ 
																					html image
																						class: 'ui-corner-all choice';
																						url: each ] ] ] ] ] ] ].
			html submitButton
				class: 'button';
				callback: [ self answer: image ];
				with: 'Continue' ]! !

!VOAlternativeMBVoteEditor methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 5/8/2012 14:33'!
style
	^ 'div.secrets img {height:70px;width:150px;}'! !

!VOMBVoteEditor class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 5/7/2012 09:31'!
choice: aChoice
	^ self new
		choice: aChoice;
		yourself! !

!VOMBVoteEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/9/2012 10:17'!
assign: value
	error := nil.
	number := value greaseInteger! !

!VOMBVoteEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/7/2012 09:22'!
choice: anObject
	choice := anObject! !

!VOMBVoteEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/7/2012 09:26'!
renderContentOn: html
	self renderDescriptionOn: html.
	self renderErrorOn: html.
	self renderFormOn: html! !

!VOMBVoteEditor methodsFor: 'rendering-parts' stamp: 'NicolasVanderAuwera 5/7/2012 09:35'!
renderDescriptionOn: html
	html
		paragraph: [ 
			html text: 'Please enter a number for the candidate: '.
			html strong: choice name.
			html text: '. It must be greater than 0.' ]! !

!VOMBVoteEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/9/2012 09:57'!
renderErrorOn: html
	error
		ifNotNil: [ :arg | 
			html paragraph
				class: 'error';
				with: error ]! !

!VOMBVoteEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/9/2012 10:17'!
renderFormOn: html
	html
		form: [ 
			html textInput callback: [ :value | self assign: value ].
			html submitButton
				callback: [ 
					number > 0
						ifTrue: [ self answer: number ]
						ifFalse: [ error := 'The number must be greater than 0.' ] ] ]! !

WALabelledFormDialog subclass: #VOBallotEditor
	instanceVariableNames: 'model methods group choices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-View'!

!VOBallotEditor class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/5/2012 07:54'!
model: aBallot choices: choicesList
	^ self new
		model: aBallot;
		choices: choicesList;
		yourself! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 13:46'!
buttons
	^ #(next)! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/5/2012 07:54'!
choices: choicesList
	choices := choicesList! !

!VOBallotEditor methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 1/7/2012 15:54'!
initialize
	super initialize.
	self initializeMethods.
	self validateWith: [ :aBallot | aBallot validate ].
	self addMessage: 'Please select your choice(s)'! !

!VOBallotEditor methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 1/22/2012 10:27'!
initializeMethods
	methods := Dictionary new
		at: #VOBallot put: #renderRadioVote:on:;
		at: #VOKOutOfNBallot put: #renderCheckBoxVote:on:;
		at: #VOMPtKOutOfNBallot put: #renderNumberInputVote:on:;
		at: #VOMBBallot put: #renderMBNumberInputVote:on:;
		yourself! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/21/2012 07:37'!
model
	^ model! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/8/2012 08:24'!
model: anObject 
	model := anObject! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 16:09'!
modelSymbol
	^ methods at: self model class name asSymbol! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/23/2012 08:06'!
next
	self answer: self model! !

!VOBallotEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 3/10/2012 10:39'!
renderCheckBoxVote: each on: html
	html checkbox
		callback: [ :value | self model give: 1 additionalPointsTo: each ];
		value: (self model scoreOf: each) isZero not! !

!VOBallotEditor methodsFor: 'rendering-parts' stamp: 'NicolasVanderAuwera 1/7/2012 16:11'!
renderDescriptionOn: html
	html paragraph: self model description! !

!VOBallotEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 3/10/2012 10:33'!
renderNumberInputVote: each on: html
	html numberInput
		callback: [ :value | self model give: value greaseInteger additionalPointsTo: each ];
		value: each score! !

!VOBallotEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 3/10/2012 10:33'!
renderRadioVote: each on: html
	group radioButton
		callback: [ self model give: 1 additionalPointsTo: each ];
		selected: (self model scoreOf: each) isZero not! !

!VOBallotEditor methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 2/20/2012 10:52'!
renderVotesOn: html
	group := html radioGroup.
	html
		table: [ 
			1 to: choices size do: [ :i | 
				html
					tableRow: [ 
						html tableData: (choices at: i) name.
						html tableData: [ self perform: self modelSymbol with:  i with: html ] ] ] ]! !

!VOBallotEditor methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 14:02'!
rows
	^ self model fields! !

GRObject variableSubclass: #VOBallot
	instanceVariableNames: 'blank numberOfPossibleVotes votes electionId createdAt voterId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Model'!

!VOBallot commentStamp: 'NicolasVanderAuwera 3/1/2012 09:50' prior: 0!
Used when the election is an "Choose 1 choice out of N" election.!

!VOBallot class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/29/2012 15:50'!
numberOfPossibleVotes: aNumber
	^ self new
		numberOfPossibleVotes: aNumber;
		yourself
	! !

!VOBallot methodsFor: 'arithmetic' stamp: 'NicolasVanderAuwera 1/26/2012 09:42'!
+ aBallot
	| ballot |
	ballot := self class new.
	ballot electionId: self electionId.
	ballot votes: self votes + aBallot votes.
	^ ballot
	! !

!VOBallot methodsFor: 'arithmetic' stamp: 'NicolasVanderAuwera 8/20/2012 11:13'!
- aBallot
	| ballot |
	ballot := self class new.
	ballot electionId: self electionId.
	ballot votes: self votes - aBallot votes.
	^ ballot! !

!VOBallot methodsFor: 'comparing' stamp: 'NicolasVanderAuwera 3/6/2012 14:39'!
= aBallot
	^ voterId = aBallot voterId and: [ electionId = aBallot electionId ]! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:52'!
assertIndex: aPosition
	self assert: aPosition < self numberOfPossibleVotes! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:54'!
assertInvariant
	self assertSize.
	self assertValues! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:53'!
assertSize
	self assert: self votes size = self numberOfPossibleVotes! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:55'!
assertValue: aVote
	self assert: (aVote isInteger and: [ aVote isZero or: [ aVote isOne ] ])! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:55'!
assertValues
	self votes do: [ :vote | self assertValue: vote ]! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/10/2012 10:27'!
availableSlots
	| slots |
	slots := OrderedCollection new.
	1 to: self votes size do: [ :slot | 
		(self choiceIsChoosable: slot)
			ifTrue: [ slots add: slot ] ].
	^ slots! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:51'!
choiceIsChoosable: aPosition
	^ (self votes at: aPosition) isZero! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/6/2012 14:40'!
createdAt
	^ createdAt! !

!VOBallot methodsFor: 'defaults' stamp: 'NicolasVanderAuwera 2/29/2012 15:42'!
defaultChoiceScore
	^ 0! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 13:52'!
description
	^ 'You have to select one choice.'! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/8/2011 07:34'!
electionId
	^ electionId! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/8/2011 07:34'!
electionId: anObject
	electionId := anObject! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 14:01'!
fields
	^ #(description votes)! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:15'!
give: anInteger additionalPointsTo: aPosition
	self assertValue: anInteger.
	self votes at: aPosition put: anInteger.
	self assertInvariant! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 3/1/2012 15:58'!
hasValidVotes
	^ (self votes select: [ :vote | vote isOne ]) size = 1! !

!VOBallot methodsFor: 'comparing' stamp: 'NicolasVanderAuwera 3/6/2012 14:39'!
hash
	"Hash is reimplemented because = is implemented."

	^ electionId hash bitXor: voterId hash! !

!VOBallot methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 8/20/2012 11:03'!
initialize
	super initialize.
	createdAt := DateAndTime now.
	blank := false! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/1/2012 15:52'!
initializeVotes
	votes := (1 to: self numberOfPossibleVotes) collect: [ :i | self defaultChoiceScore ]! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/8/2012 13:20'!
isAlternativeBallot
	^ false! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 19:42'!
isBallot
	^ true! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 8/20/2012 11:03'!
isBlank
	^ blank! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:38'!
isMBBallot
	^ false! !

!VOBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:28'!
isRemainingVotes
	^ (self votes select: [ :vote | vote isOne ]) size < 1! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/29/2012 08:06'!
numberOfPossibleVotes
	^ numberOfPossibleVotes! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 15:50'!
numberOfPossibleVotes: anInteger
	self assert: anInteger > 0.
	numberOfPossibleVotes := anInteger! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 15:43'!
scoreOf: anInteger
	^ self votes at: anInteger! !

!VOBallot methodsFor: 'validation' stamp: 'NicolasVanderAuwera 1/8/2012 07:55'!
validate
	self hasValidVotes
		ifFalse: [ WAValidationNotification signal: 'You can not select more than one choice.' ]! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/6/2012 14:26'!
voterId
	^ voterId! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/6/2012 14:26'!
voterId: anObject
	voterId := anObject! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 15:44'!
votes
	votes ifNil: [ self initializeVotes ].
	^ votes! !

!VOBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/22/2012 11:40'!
votes: votesArray
	votes := votesArray! !

VOBallot subclass: #VOKOutOfNBallot
	instanceVariableNames: 'maxNumberOfAllowedVotes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Model'!

!VOKOutOfNBallot commentStamp: 'NicolasVanderAuwera 11/12/2011 10:00' prior: 0!
Used when the election is an "Choose K choices out of N" election!

!VOKOutOfNBallot class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/29/2012 16:10'!
numberOfPossibleVotes: aNumber maxNumberOfAllowedVotes: maxNumber
	^ self new
		numberOfPossibleVotes: aNumber;
		maxNumberOfAllowedVotes: maxNumber;
		yourself! !

!VOKOutOfNBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:15'!
assertValue: aVote
	self assert: (aVote isInteger and: [ aVote >= self defaultChoiceScore ])! !

!VOKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 13:53'!
description
	^ 'You must still select ' , self maxNumberOfAllowedVotes greaseString , ' choices.'! !

!VOKOutOfNBallot methodsFor: 'validation' stamp: 'NicolasVanderAuwera 2/29/2012 16:12'!
hasValidMaxNumberOfAllowedVotes
	^ (self votes select: [ :vote | vote isOne ]) size <= self maxNumberOfAllowedVotes! !

!VOKOutOfNBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 1/8/2012 07:56'!
hasValidVotes
	^ true! !

!VOKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/20/2011 09:18'!
maxNumberOfAllowedVotes
	^ maxNumberOfAllowedVotes! !

!VOKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:11'!
maxNumberOfAllowedVotes: maxNumber
	self assert: maxNumber > 0.
	self assert: maxNumber <= self numberOfPossibleVotes.
	maxNumberOfAllowedVotes := maxNumber! !

!VOKOutOfNBallot methodsFor: 'validation' stamp: 'NicolasVanderAuwera 1/8/2012 07:57'!
validate
	super validate.
	self hasValidMaxNumberOfAllowedVotes
		ifFalse: [ WAValidationNotification signal: 'You select too many choices.' ]! !

VOKOutOfNBallot subclass: #VOMPtKOutOfNBallot
	instanceVariableNames: 'pointsToDistribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Model'!

!VOMPtKOutOfNBallot class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/29/2012 16:25'!
numberOfPossibleVotes: aNumber maxNumberOfAllowedVotes: maxNumber pointsToDistribute: points
	^ self new
		numberOfPossibleVotes: aNumber;
		maxNumberOfAllowedVotes: maxNumber;
		pointsToDistribute: points greaseInteger;
		yourself! !

!VOMPtKOutOfNBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:21'!
assertAllowedAndUsedPoints
	self assert: self pointsAttributed + self remainingPoints = self pointsToDistribute! !

!VOMPtKOutOfNBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:21'!
assertInvariant
	super assertInvariant.
	self assertAllowedAndUsedPoints! !

!VOMPtKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/7/2012 13:54'!
description
	^ 'You have ' , self remainingPoints greaseString , ' points to distribute.'! !

!VOMPtKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:21'!
give: anInteger additionalPointsTo: aPosition
	self assertAllowedAndUsedPoints.
	super give: anInteger additionalPointsTo: aPosition.
	! !

!VOMPtKOutOfNBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/21/2011 13:34'!
isPointsAttributedGreaterOrEqualZero
	^ self remainingPoints >= 0! !

!VOMPtKOutOfNBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 3/10/2012 10:24'!
isRemainingPointsToDistribute
	^ self remainingPoints > 0! !

!VOMPtKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/22/2012 11:54'!
pointsAttributed
	^ self votes sum! !

!VOMPtKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/20/2011 07:29'!
pointsToDistribute
	^ pointsToDistribute! !

!VOMPtKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:25'!
pointsToDistribute: aNumber
	self assert: aNumber > 0.
	pointsToDistribute := aNumber! !

!VOMPtKOutOfNBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/22/2012 11:54'!
remainingPoints
	^ self pointsToDistribute - self pointsAttributed! !

!VOMPtKOutOfNBallot methodsFor: 'validation' stamp: 'NicolasVanderAuwera 1/8/2012 07:57'!
validate
	super validate.
	self isPointsAttributedGreaterOrEqualZero
		ifFalse: [ WAValidationNotification signal: 'You gave too many points.' ]
	! !

VOBallot variableSubclass: #VOMBBallot
	instanceVariableNames: 'verified signatures'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Model'!

VOMBBallot subclass: #VOMBAlternativeBallot
	instanceVariableNames: 'secrets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Model'!

!VOMBAlternativeBallot class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 2/29/2012 16:46'!
numberOfPossibleVotes: aNumber secrets: secretsList
	^ self new
		numberOfPossibleVotes: aNumber;
		yourself! !

!VOMBAlternativeBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 8/4/2012 08:37'!
assertInvariant
	self assertSize! !

!VOMBAlternativeBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/8/2012 07:36'!
fields
	^ super fields , #(secrets)! !

!VOMBAlternativeBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/4/2012 08:36'!
give: aString additionalPointsTo: aPosition
	self votes at: aPosition put: aString.
	self assertInvariant! !

!VOMBAlternativeBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:47'!
hasValidSecrets
	^ self votes allSatisfy: [ :vote | self secrets includes: vote ]! !

!VOMBAlternativeBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:48'!
hasVoteWithSecret: secret
	^ self votes anySatisfy: [ :vote | vote = secret ]! !

!VOMBAlternativeBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:50'!
isAValidSecret: aSecret
	^ self secrets anySatisfy: [ :s | aSecret = s ]! !

!VOMBAlternativeBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/8/2012 13:20'!
isAlternativeBallot
	^ true! !

!VOMBAlternativeBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/8/2012 13:21'!
isMBBallot
	^ false! !

!VOMBAlternativeBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:47'!
secrets
	^ secrets! !

!VOMBAlternativeBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:47'!
secrets: secretsList
	secrets := secretsList! !

!VOMBAlternativeBallot methodsFor: 'adding' stamp: 'NicolasVanderAuwera 2/29/2012 16:50'!
verifySecret: aSecret
	(self isAValidSecret: aSecret)
		ifFalse: [ WAValidationNotification signal: 'The secret you chosen is not included in the set of available secrets' ].
	(self hasVoteWithSecret: aSecret)
		ifTrue: [ WAValidationNotification signal: 'You already give this secret to another vote' ]! !

!VOMBBallot class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 1/23/2012 11:11'!
from: aBallot
	^ self new
		electionId: aBallot electionId;
		votes: aBallot votes;
		yourself! !

!VOMBBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:26'!
assertValue: aVote
	self assert: (aVote isInteger and: [ aVote >= 0 ])! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/5/2012 15:36'!
blank: bool
	blank := bool! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/8/2012 08:03'!
description
	^ 'You have to put a random number in the text input.'! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/29/2012 16:31'!
emptySlots
	"return positions in votes array where there is a zero"

	| emptySlots |
	emptySlots := OrderedCollection new.
	1 to: self votes size do: [ :i | 
		(self votes at: i) isZero
			ifTrue: [ emptySlots add: i ] ].
	^ emptySlots! !

!VOMBBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:44'!
hasDuplicates
	^ self hasDuplicatesBefore: self votes size! !

!VOMBBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 16:43'!
hasDuplicatesBefore: aPosition
	self assert: aPosition <= self votes size.
	1 to: aPosition do: [ :i | 
		i + 1 to: aPosition do: [ :j | 
			(self votes at: i) = (self votes at: j)
				ifTrue: [ ^ true ] ] ].
	^ false! !

!VOMBBallot methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 8/20/2012 11:03'!
initialize
	super initialize.
	verified := false! !

!VOMBBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:38'!
isMBBallot
	^ true! !

!VOMBBallot methodsFor: 'testing' stamp: 'NicolasVanderAuwera 3/1/2012 15:24'!
isRemainingVotes
	^ (self votes select: [ :vote | vote isZero ]) notEmpty! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/23/2012 14:19'!
signatures
	^ signatures! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/23/2012 14:19'!
signatures: aList
	signatures := aList! !

!VOMBBallot methodsFor: 'validation' stamp: 'NicolasVanderAuwera 2/29/2012 16:38'!
validate
	self isRemainingVotes
		ifTrue: [ WAValidationNotification signal: 'You have to fill all votes.' ].
	self hasDuplicates
		ifTrue: [ ^ WAValidationNotification signal: 'You cannot enter two same secrets.' ]! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/21/2012 09:09'!
verified
	^ verified! !

!VOMBBallot methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/21/2012 09:09'!
verified: anObject
	verified := anObject! !

GRObject subclass: #VOUrn
	instanceVariableNames: 'ballots id closed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-Urn-Model'!

!VOUrn commentStamp: 'NicolasVanderAuwera 12/7/2011 10:40' prior: 0!
Urn og the election. It contains multiple ballots. The id variable must be the same as the id of the election.!

!VOUrn class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 12/8/2011 08:06'!
id: anId
	^ self new
		id: anId;
		yourself! !

!VOUrn methodsFor: 'adding' stamp: 'NicolasVanderAuwera 12/9/2011 07:13'!
<= anUrn
	^ self id <= anUrn id! !

!VOUrn methodsFor: 'adding' stamp: 'NicolasVanderAuwera 3/29/2012 13:28'!
addBallot: aBallot
	ballots add: aBallot.
	self assertInvariant! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 19:44'!
assertAllBallotsCorrectType
	ballots do: [ :each | self assert: each isBallot ]! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 19:42'!
assertAllBallotsSameSize
	| size |
	self isEmpty
		ifTrue: [ ^ self ].
	size := ballots first numberOfPossibleVotes.
	ballots do: [ :each | self assert: each numberOfPossibleVotes = size ]! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/29/2012 19:42'!
assertInvariant
	self assertAllBallotsCorrectType.
	self assertAllBallotsSameSize! !

!VOUrn methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/7/2012 14:47'!
ballotDesign
	^ ballots first class name! !

!VOUrn methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/7/2011 18:12'!
ballots
	^ ballots! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 12/9/2011 07:20'!
close
	closed := true.
	self triggerEvent: #close! !

!VOUrn methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/21/2012 14:45'!
compressed: factor
	"Creates a new urn with elements summed factor by factor. Example: Urn: 1|2|3|4|5|6 compressed with a factor 2 gives us a new Urn with the ballots 1+2|3+4|5+6 = 3|7|11"

	| newUrn i b copy |
	newUrn := self class new.
	i := 0.
	[ i + factor - 1 < ballots size ]
		whileTrue: [ 
			b := ballots at: i.
			1 to: factor do: [ :j | b := b + ballots at: i + j ].
			newUrn addBallot: b.
			i := i + factor ].
	[ i < ballots size ]
		whileTrue: [ 
			copy := (ballots at: i) deepCopy.
			newUrn addBallot: copy.
			i := i + 1 ]! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 2/26/2012 14:26'!
createSample
	| random ballot |
	random := SecureRandom new.
	(random nextInt: 1000)
		timesRepeat: [ 
			ballot := VOBallot numberOfPossibleVotes: (random nextInt: 100).
			ballot createSample.
			self addBallot: ballot ]! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 12/7/2011 10:38'!
id
	^ id! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 12/7/2011 10:38'!
id: anInteger
	id := anInteger! !

!VOUrn methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 12/9/2011 07:04'!
initialize
	super initialize.
	ballots := OrderedCollection new.
	closed := false! !

!VOUrn methodsFor: 'tests' stamp: 'NicolasVanderAuwera 12/9/2011 07:03'!
isClosed
	^ closed! !

!VOUrn methodsFor: 'testing' stamp: 'NicolasVanderAuwera 11/7/2011 16:06'!
isEmpty
	^ ballots isEmpty! !

!VOUrn methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 09:26'!
numberOfChoices
	self isEmpty
		ifTrue: [ ^ 0 ].
	^ ballots first numberOfPossibleVotes! !

!VOUrn methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 11/7/2011 16:07'!
tally
	^ ballots size! !

!VOUrn methodsFor: 'removing' stamp: 'NicolasVanderAuwera 3/6/2012 16:45'!
withoutDuplicate
	| ballotsCopy ballotsWithoutDuplicate |
	ballotsCopy := self ballots copy.
	ballotsWithoutDuplicate := OrderedCollection new.
	1 to: ballotsCopy size do: [ :i | 
		| laterBallot |
		laterBallot := ballotsCopy at: i ifAbsent: [ nil ].
		laterBallot notNil
			ifTrue: [ 
				i + 1 to: ballotsCopy size do: [ :j | 
					| aBallot |
					aBallot := ballotsCopy at: j ifAbsent: [ nil ].
					aBallot notNil
						ifTrue: [ 
							laterBallot = aBallot
								ifTrue: [ 
									laterBallot createdAt < aBallot createdAt
										ifTrue: [ 
											ballotsCopy remove: laterBallot.
											laterBallot := aBallot ]
										ifFalse: [ ballotsCopy remove: aBallot ifAbsent: [  ] ] ] ] ] ] ].
	^ ballots! !
